@@ -7,17 +7,17 @@ try:
     from .inspyred.ea import EA as InspyredEA
     engines['inspyred'] = InspyredEA
 except ImportError:
-    pass
+    print("inspyred not available")
 
 try:
     from .jmetal.ea import EA as JMetalEA
     engines['jmetal'] = JMetalEA
 except ImportError:
-    pass
+    print("jmetal not available")
 
 
-algorithms ={ 'inspyred' : ['NSGAII'],
-              'jmetal': ['NSGAII','SPEA2','NSGAIII']
+algorithms ={ 'inspyred' : ['SA','GA','NSGAII'],
+              'jmetal'   : ['SA','GA','NSGAII','SPEA2','NSGAIII']
             }
 
 
@@ -46,10 +46,8 @@ def get_default_engine():
 
 def set_default_engine(enginename):
     """ Sets default EA engine.
-    
-    Parameters:
-    
-    enginename : (str) engine name (currently available: 'inspyred', 'jmetal')
+   
+    :param str enginename: Optimization engine (currently available: 'inspyred', 'jmetal')
     """
 
     global default_engine
@@ -61,7 +59,10 @@ def set_default_engine(enginename):
 
 
 def set_preferred_EA(algorithm):
-    "Defines de preferred MOEA."
+    """Defines de preferred MOEA.
+    
+    :param str algorithm: The name of the preferred algorithm.
+    """
     global preferred_EA
     global default_engine
 
@@ -77,35 +78,49 @@ def set_preferred_EA(algorithm):
 
 
 def get_preferred_EA():
+    """
+    :returns: The name of the preferred MOEA.
+    """
     global preferred_EA
     return preferred_EA
 
 
 def get_available_engines():
+    """
+    :returns: The list of available engines.
+    """
     return list(engines.keys())
 
 
-
-
-def EA(problem, initial_population=[], max_generations=EAConstants.MAX_GENERATIONS, mp=True, visualizer=False):
+def get_available_algorithms():
     """
-    EA running helper
+    :returns: The list of available MOEAs.
+    """
+    l = set()
+    for engine in engines.keys():
+        l.union(set(engines[engine]))
+    return list(l)
 
-    Parameters:
-    
-    problem: the optimization problem
-    initial_population* (list): the EA initial population
-    max_generations (int): the number of iterations of the EA (stopping criteria)
-    mp (boolean): if should use multiprocessing
-    visualizer (boolean): if the pareto font should be displayed. Requires a graphic environment.
-    
-    Returns:
-    
-    An instance of an EA optimizer.
+
+
+def EA(problem, initial_population=[], max_generations=EAConstants.MAX_GENERATIONS, mp=True, visualizer=False, algorithm = None ):
+    """
+    EA running helper. Returns an instance of the EA that reflects the global user configuration settings such as preferred engine and algorithm.
+
+    :param problem: The optimization problem.
+    :param list initial_population: The EA initial population.
+    :param int max_generations: The number of iterations of the EA (stopping criteria).
+    :param bool mp: If multiprocessing should be used. 
+    :param bool visualizer: If the pareto font should be displayed. Requires a graphic environment.
+    :returns: An instance of an EA optimizer.
     
     """
 
     if len(engines) == 0:
         raise RuntimeError('Inspyred or JMetal packages are required.')
     engine = engines[get_default_engine()]
-    return engine(problem, initial_population=initial_population, max_generations=max_generations, mp=mp, visualizer=visualizer,algorithm=get_preferred_EA())
+    
+    if algorithm is None or algorithm not in get_available_algorithms():
+        algorithm=get_preferred_EA()
+     
+    return engine(problem, initial_population=initial_population, max_generations=max_generations, mp=mp, visualizer=visualizer,algorithm=algorithm)
