@@ -14,13 +14,17 @@ from mewpy.utils.constants import ModelConstants
 from mewpy.utils.parsing import evaluate_expression_tree
 from collections import OrderedDict
 import numpy as np
+import logging
+
+
+LOGGER = logging.getLogger(__name__)
 
 
 class CBModelContainer(ModelContainer):
     """ A basic container for REFRAMED models.
-    
+
     :param model: A metabolic model.
-    
+
     """
     def __init__(self, model: CBModel):
         if not isinstance(model, CBModel):
@@ -46,10 +50,10 @@ class CBModelContainer(ModelContainer):
 
     def get_gpr(self, reaction_id):
         """Returns the gpr rule (str) for a given reaction ID.
-        
+
         :param str reaction_id: The reaction identifier.
         :returns: A string representation of the GPR rule.
-        
+
         """
         if reaction_id not in self.reactions:
             raise ValueError(f"Reactions {reaction_id} does not exist")
@@ -88,17 +92,17 @@ class CBModelContainer(ModelContainer):
 class Simulation(CBModelContainer, Simulator):
     """Generic Simulation class for cobra Model.
        Defines the simulation conditions, and makes available a set of methods.
-       
+
     :param model: An metabolic model instance.
-    
+
     Optional:
-    
+
     :param objective: The model objective.
     :param dic envcond: Dictionary of environmental conditions.
-    :param dic constraints: A dictionary of reaction contraints.
+    :param dic constraints: A dictionary of reaction constraints.
     :param solver: An instance of the LP solver.
     :param dic reference: A dictionary of the wild type flux values.
-    
+
     """
 
     def __init__(self, model: CBModel, objective=None, envcond=None, constraints=None,  solver=None, reference=None):
@@ -109,7 +113,7 @@ class Simulation(CBModelContainer, Simulator):
         self.model = model
         try:
             self.objective = self.model.get_objective() if objective is None else objective
-        except:
+        except Exception:
             self.objective = None
         self.environmental_conditions = OrderedDict() if envcond is None else envcond
         self.constraints = OrderedDict() if constraints is None else constraints
@@ -136,9 +140,9 @@ class Simulation(CBModelContainer, Simulator):
     @property
     def reference(self):
         """The reference wild type reaction flux values.
-        
+
         :returns: A dictionary of wild type reaction flux values.
-        
+
         """
         if self._reference is None:
             self._reference = self.simulate(
@@ -147,11 +151,12 @@ class Simulation(CBModelContainer, Simulator):
 
     @property
     def essential_reactions(self, min_growth=0.01):
-        """Essential reactions are those when knocked out enable a biomass flux value above a minimal growth defined as a percentage of the wild type growth.
-        
+        """Essential reactions are those when knocked out enable a biomass flux value above a minimal growth defined as
+        a percentage of the wild type growth.
+
         :param float min_growth: Minimal percentage of the wild type growth value. Default 0.01 (1%).
         :returns: A list of essential reactions.
-        
+
         """
         if self._essential_reactions is not None:
             return self._essential_reactions
@@ -162,17 +167,19 @@ class Simulation(CBModelContainer, Simulator):
         for rxn in reactions:
             res = self.simulate(constraints={rxn: 0})
             if res:
-                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) or res.status == SStatus.INFEASIBLE:
+                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) \
+                        or res.status == SStatus.INFEASIBLE:
                     self._essential_reactions.append(rxn)
         return self._essential_reactions
 
     @property
     def essential_genes(self, min_growth=0.01):
-        """Essential renes are those when deleted enable a biomass flux value above a minimal growth defined as a percentage of the wild type growth.
-        
+        """Essential genes are those when deleted enable a biomass flux value above a minimal growth defined as
+        a percentage of the wild type growth.
+
         :param float min_growth: Minimal percentage of the wild type growth value. Default 0.01 (1%).
         :returns: A list of essential genes.
-        
+
         """
         if self._essential_genes is not None:
             return self._essential_genes
@@ -188,16 +195,17 @@ class Simulation(CBModelContainer, Simulator):
             gr_constraints = {rxn: 0 for rxn in inactive_reactions}
             res = self.simulate(constraints=gr_constraints)
             if res:
-                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) or res.status == SStatus.INFEASIBLE:
+                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) \
+                        or res.status == SStatus.INFEASIBLE:
                     self._essential_genes.append(gene)
         return self._essential_genes
 
     def evaluate_gprs(self, active_genes):
         """Returns the list of active reactions for a given list of active genes.
-        
-        :param list active_genes: List of genes idenfiers.
+
+        :param list active_genes: List of genes identifiers.
         :returns: A list of active reaction identifiers.
-        
+
         """
         active_reactions = []
         reactions = self.model.reactions
@@ -209,12 +217,11 @@ class Simulation(CBModelContainer, Simulator):
                 active_reactions.append(r_id)
         return active_reactions
 
-
     def update(self):
         self.model.update()
 
-    def add_reaction(self,reaction,replace=True):
-        self.model.add_reaction(reaction,replace=replace)    
+    def add_reaction(self, reaction, replace=True):
+        self.model.add_reaction(reaction, replace=replace)
 
     def remove_reaction(self, r_id):
         self.model.remove_reaction(r_id)
@@ -229,28 +236,29 @@ class Simulation(CBModelContainer, Simulator):
 
     def get_uptake_reactions(self):
         """
-        
         :returns: The list of uptake reactions.
-        
-        """        
+
+        """
         drains = self.get_drains()
         reacs = [r for r in drains if self.model.reactions[r].reversible or
-                 ((self.model.reactions[r].lb is None or self.model.reactions[r].lb < 0)and len(self.model.reactions[r].get_substrates()) > 0) or
-                 ((self.model.reactions[r].ub is None or self.model.reactions[r].ub > 0)and len(self.model.reactions[r].get_products())) > 0]
+                 ((self.model.reactions[r].lb is None or self.model.reactions[r].lb < 0)
+                  and len(self.model.reactions[r].get_substrates()) > 0) or
+                 ((self.model.reactions[r].ub is None or self.model.reactions[r].ub > 0)
+                  and len(self.model.reactions[r].get_products())) > 0]
         return reacs
 
     def reverse_reaction(self, reaction_id):
         """
-        Identify if a reaction is reversible and returns the 
+        Identify if a reaction is reversible and returns the
         reverse reaction if it is the case.
-        
+
         :param reaction_id: A reaction identifier.
         :return: A reverse reaction identifier or None
-        
+
         """
-        
-        #TODO: ... use regex instead  
-        
+
+        # TODO: ... use regex instead.
+
         rxn = self.model.reactions[reaction_id]
         reactions = self.model.reactions
         if rxn.lb < 0:
@@ -274,7 +282,7 @@ class Simulation(CBModelContainer, Simulator):
     def gene_reactions(self):
         """
         :returns: a map of genes to reactions.
-        
+
         """
         if not self._gene_to_reaction:
             gr = OrderedDict()
@@ -293,9 +301,9 @@ class Simulation(CBModelContainer, Simulator):
     def get_reactions_for_genes(self, genes):
         """
         Returns the list of reactions catalysed by a list of genes
-       
+
         :param list genes: A list of gene IDs.
-        :returns: A list of reaction identifieres.
+        :returns: A list of reaction identifiers.
 
         """
         if not self._gene_to_reaction:
@@ -308,21 +316,20 @@ class Simulation(CBModelContainer, Simulator):
     def get_reaction_metabolites(self, reaction):
         '''
         Returns all metabolites of a given reaction
-        
+
         :param reaction: reaction (str)
         :return: metabolites (dict)
-        
+
         '''
         return self.model.reactions[reaction].stoichiometry
 
     def is_reactant(self, reaction, metabolite):
         '''
         Returns if a metabolite is reactant into a given reaction
-        
+
         :param reaction: reaction (str)
         :param metabolite: metabolite (str)
         :return: bool
-        
         '''
         if metabolite not in self.model.reactions[reaction].stoichiometry:
             raise KeyError("{} not in {}".format(metabolite, reaction))
@@ -330,7 +337,8 @@ class Simulation(CBModelContainer, Simulator):
 
     def is_product(self, reaction, metabolite):
         '''
-        Returns if a metabolite is product into a given reaction
+        Returns if a metabolite is product into a given reaction.
+
         :param reaction: reaction (str)
         :param metabolite: metabolite (str)
         :return: bool
@@ -350,9 +358,8 @@ class Simulation(CBModelContainer, Simulator):
                 else:
                     self._index_metabolites_reactions[metabolite] = [reaction]
 
-
     def metabolite_reaction_lookup(self, force_recalculate=False):
-        """ Return the network topology as a nested map from metabolite to reaction to coefficient
+        """ Return the network topology as a nested map from metabolite to reaction to coefficient.
         :return: a dictionary lookup table
         """
 
@@ -365,16 +372,12 @@ class Simulation(CBModelContainer, Simulator):
 
         return self._m_r_lookup
 
-
-    
     def set_objective(self, reaction):
         self.model.set_objective({reaction: 1})
 
-    
     def get_objective(self):
         return list(self.objective.keys())
 
-
     def get_S(self):
         """
         Returns the S matrix as a numpy array
@@ -383,9 +386,7 @@ class Simulation(CBModelContainer, Simulator):
 
         return np.array(self.model.stoichiometric_matrix())
 
-
     def get_reaction_bounds(self, reaction):
-
         """
         Returns the bounds for a given reaction.
         :param reaction: str, reaction ID
@@ -401,8 +402,6 @@ class Simulation(CBModelContainer, Simulator):
 
         return lb if lb > -np.inf else -999999, ub if ub < np.inf else 999999
 
-
-
     def get_bounds(self):
         """
         Returns the whole set of lower and upper bounds as numpy arrays
@@ -410,10 +409,34 @@ class Simulation(CBModelContainer, Simulator):
         :return: lb(s), ub(s), tuple of lists
 
         """
-
         lbs, ubs = list(zip(*[self.get_reaction_bounds(reaction) for reaction in self.model.reactions]))
         return list(lbs), list(ubs)
 
+    def find_bounds(self):
+        """
+        Return the median upper and lower bound of the metabolic model.
+        Bounds can vary from model to model. Cobrapy defaults to (-1000, 1000).
+        """
+        lower_bounds = np.asarray([rxn.lb for rxn in self.model.reactions], dtype=float)
+        upper_bounds = np.asarray([rxn.ub for rxn in self.model.reactions], dtype=float)
+        lower_bound = np.nanmedian(lower_bounds[lower_bounds != 0.0])
+        upper_bound = np.nanmedian(upper_bounds[upper_bounds != 0.0])
+        if np.isnan(lower_bound):
+            LOGGER.warning("Could not identify a median lower bound.")
+            lower_bound = -1000.0
+        if np.isnan(upper_bound):
+            LOGGER.warning("Could not identify a median upper bound.")
+            upper_bound = 1000.0
+        return lower_bound, upper_bound
+        
+    def find_unconstrained_reactions(self):
+        """Return list of reactions that are not constrained at all."""
+        lower_bound, upper_bound = self.find_bounds()
+        return [
+            rxn
+            for rxn in self.model.reactions
+            if rxn.lb <= lower_bound and rxn.ub >= upper_bound
+        ]
 
 
     def get_boundary_reaction(self, metabolite):
@@ -438,22 +461,20 @@ class Simulation(CBModelContainer, Simulator):
 
         return None
 
-    
-    
-    
-    
-    def simulate(self, objective=None, method=SimulationMethod.FBA, maximize=True, constraints=None, reference=None, scalefactor=None, solver=None):
+    # Simulate
+    def simulate(self, objective=None, method=SimulationMethod.FBA,
+                 maximize=True, constraints=None, reference=None,
+                 scalefactor=None, solver=None):
         '''
         Simulates a phenotype when applying a set constraints using the specified method.
 
-        
         :param dic objective: The simulation objective. If none, the model objective is considered.
         :param method: The SimulationMethod (FBA, pFBA, lMOMA, etc ...)
         :param boolean maximize: The optimization direction
-        :param dic contraints: A dictionary of contraints to be applied to the model.
-        :param dic reference: A dictionary of reaction flux values. 
+        :param dic constraints: A dictionary of constraints to be applied to the model.
+        :param dic reference: A dictionary of reaction flux values.
         :param float scalefactor: A positive scaling factor for the solver. Default None.
-         
+        :param solver: An instance of the solver.
         '''
 
         a_solver = solver
@@ -519,7 +540,7 @@ class Simulation(CBModelContainer, Simulator):
             raise Exception(
                 "Unknown method to perform the simulation.")
 
-        # undoes the model scalling
+        # undoes the model scaling
         if scalefactor:
             for _, rxn in self.model.reactions.items():
                 rxn.lb = rxn.lb / scalefactor
@@ -538,7 +559,7 @@ class Simulation(CBModelContainer, Simulator):
 
     def FVA(self, obj_frac=0.9, reactions=None, constraints=None, loopless=False, internal=None, solver=None):
         """ Flux Variability Analysis (FVA).
-        
+
         :param model: An instance of a constraint-based model.
         :param float obj_frac: The minimum fraction of the maximum growth rate (default 0.9). Requires that the objective value is at least the fraction times maximum objective value. A value of 0.85 for instance means that the objective has to be at least at 85% percent of its maximum.
         :param list reactions: List of reactions to analyze (default: all).
@@ -547,13 +568,16 @@ class Simulation(CBModelContainer, Simulator):
         :param list internal: List of internal reactions for looplessFBA (optional).
         :param solver: A pre-instantiated solver instance (optional)
         :returns: A dictionary of flux variation ranges.
-        
+
         """
+        _constraints = {}
+        if constraints:
+            _constraints.update(constraints)
+        if self.environmental_conditions:
+            _constraints.update(self.environmental_conditions)
         from reframed.cobra.variability import FVA
         return FVA(self.model, obj_frac=obj_frac, reactions=reactions,
-                   constraints=constraints, loopless=loopless, internal=internal, solver=solver)
-
-    
+                   constraints=_constraints, loopless=loopless, internal=internal, solver=solver)
 
 
 class GeckoSimulation(Simulation):
@@ -571,7 +595,6 @@ class GeckoSimulation(Simulation):
     def protein_rev_reactions(self):
         return self.model.protein_rev_reactions
 
-    
     def essential_proteins(self, protein_prefix, min_growth=0.01):
         if self._essential_proteins is not None:
             return self._essential_proteins
@@ -580,14 +603,14 @@ class GeckoSimulation(Simulation):
         self._essential_proteins = []
         proteins = self.model.proteins
         for p in proteins:
-            rxn = "{}{}".format(protein_prefix,p)
+            rxn = "{}{}".format(protein_prefix, p)
             res = self.simulate(constraints={rxn: 0})
             if res:
-                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) or res.status == SStatus.INFEASIBLE:
+                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) \
+                        or res.status == SStatus.INFEASIBLE:
                     self._essential_proteins.append(rxn)
         return self._essential_proteins
 
-
     def protein_reactions(self, protein):
         """
         Returns the list of reactions associated to a protein
@@ -602,11 +625,10 @@ class GeckoSimulation(Simulation):
 
     def reverse_reaction(self, reaction_id):
         """
-        Identify if a reaction is reversible and returns the 
-        reverse reaction if it is the case
+        Identify if a reaction is reversible and returns the
+        reverse reaction if it is the case.
 
-        Returns 
-            reaction identifier or None
+        :returns: A reaction identifier or None.
         """
         f, d = zip(*self.model.protein_rev_reactions.values())
         if reaction_id in f:
@@ -616,6 +638,3 @@ class GeckoSimulation(Simulation):
         else:
             return None
 
-    @staticmethod
-    def __call__(model):
-        return GeckoModel(model)
